// src/formbuilder.tsx
import { useMemo } from "react";
import {
  get,
  useController,
  useFieldArray,
  useFormState,
  useForm,
  useWatch
} from "react-hook-form";
function createFormBuilder(methods, path, key) {
  const currentPath = path.join(".");
  const cache = {};
  const { control } = methods;
  return new Proxy(
    (options) => {
      if (currentPath.length === 0) {
        throw new Error("Cannot call register at the root.");
      }
      return methods.register(currentPath, options);
    },
    {
      get(_, prop, receiver) {
        let useCached = cache[prop];
        if (useCached !== void 0) {
          return useCached;
        }
        switch (prop) {
          case Symbol.toPrimitive:
            useCached = () => currentPath;
            break;
          case "$key":
            return key ?? currentPath;
          case "$useFieldArray":
            useCached = (props) => {
              const { fields, ...rest } = useFieldArray({
                name: currentPath,
                keyName: "$key",
                control,
                ...props
              });
              return {
                fields: fields.map(
                  ({ $key }, i) => createFormBuilder(methods, [...path, i.toString()], $key)
                ),
                ...rest
              };
            };
            break;
          case "$useController":
            useCached = (props) => useController({
              name: currentPath,
              control,
              ...props
            });
            break;
          case "$useWatch":
            useCached = ({ name, ...rest } = {}) => useWatch({
              name: name != null ? prependCurrentPath(currentPath, name) : currentPath === "" ? void 0 : currentPath,
              control,
              ...rest
            });
            break;
          case "$useState":
            useCached = (props) => {
              const { errors: rootErrors, dirtyFields: rootDirtyFields } = useFormState({ ...props, control, name: currentPath });
              const errors = get(rootErrors, currentPath);
              const dirty = get(rootDirtyFields, currentPath);
              return { errors, dirty };
            };
            break;
          case "$setValue":
            useCached = (value) => {
              methods.setValue(currentPath, value);
            };
            break;
          case "$setError":
            useCached = (value, options) => {
              methods.setError(currentPath, value, options);
            };
            break;
          case "$discriminate":
            useCached = (fieldName) => [
              methods.watch(
                prependCurrentPath(
                  currentPath,
                  fieldName
                )
              ),
              receiver
            ];
            break;
          default:
            useCached = createFormBuilder(methods, [
              ...path,
              prop.toString()
            ]);
        }
        return cache[prop] = useCached;
      }
    }
  );
}
var prependCurrentPath = (currentPath, name) => {
  if (currentPath.length === 0) {
    return name ?? "";
  }
  if (typeof name === "string") {
    return `${currentPath}.${name}`;
  } else if (Array.isArray(name)) {
    return name.map((s) => `${currentPath}.${s}`);
  }
  return currentPath;
};
function useFormBuilder(props) {
  const methods = useForm(props);
  const fields = useMemo(
    () => createFormBuilder(methods, []),
    [methods]
  );
  return { fields, ...methods };
}
export {
  createFormBuilder,
  useFormBuilder
};
